# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CustomerDetails(UncheckedBaseModel):
    """
    Details about the customer making the payment.
    """

    customer_initiated: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether the customer initiated the payment.
    """

    seller_keyed_in: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates that the seller keyed in payment details on behalf of the customer.
    This is used to flag a payment as Mail Order / Telephone Order (MOTO).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..types.invoice_sort_field import InvoiceSortField
import typing_extensions
from ..types.sort_order import SortOrder


class InvoiceSortParams(typing_extensions.TypedDict):
    """
    Identifies the sort field and sort order.
    """

    field: InvoiceSortField
    """
    The field to use for sorting.
    See [InvoiceSortField](#type-invoicesortfield) for possible values
    """

    order: typing_extensions.NotRequired[SortOrder]
    """
    The order to use for sorting the results.
    See [SortOrder](#type-sortorder) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
from .subscription_event_subscription_event_type import SubscriptionEventSubscriptionEventType
import typing
from .subscription_event_info import SubscriptionEventInfo
from .phase import Phase
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SubscriptionEvent(UncheckedBaseModel):
    """
    Describes changes to a subscription and the subscription status.
    """

    id: str = pydantic.Field()
    """
    The ID of the subscription event.
    """

    subscription_event_type: SubscriptionEventSubscriptionEventType = pydantic.Field()
    """
    Type of the subscription event.
    See [SubscriptionEventSubscriptionEventType](#type-subscriptioneventsubscriptioneventtype) for possible values
    """

    effective_date: str = pydantic.Field()
    """
    The `YYYY-MM-DD`-formatted date (for example, 2013-01-15) when the subscription event occurred.
    """

    monthly_billing_anchor_date: typing.Optional[int] = pydantic.Field(default=None)
    """
    The day-of-the-month the billing anchor date was changed to, if applicable.
    """

    info: typing.Optional[SubscriptionEventInfo] = pydantic.Field(default=None)
    """
    Additional information about the subscription event.
    """

    phases: typing.Optional[typing.List[Phase]] = pydantic.Field(default=None)
    """
    A list of Phases, to pass phase-specific information used in the swap.
    """

    plan_variation_id: str = pydantic.Field()
    """
    The ID of the subscription plan variation associated with the subscription.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.weekday import Weekday


class WorkweekConfigParams(typing_extensions.TypedDict):
    """
    Sets the day of the week and hour of the day that a business starts a
    workweek. This is used to calculate overtime pay.
    """

    id: typing_extensions.NotRequired[str]
    """
    The UUID for this object.
    """

    start_of_week: Weekday
    """
    The day of the week on which a business week starts for
    compensation purposes.
    See [Weekday](#type-weekday) for possible values
    """

    start_of_day_local_time: str
    """
    The local time at which a business week starts. Represented as a
    string in `HH:MM` format (`HH:MM:SS` is also accepted, but seconds are
    truncated).
    """

    version: typing_extensions.NotRequired[int]
    """
    Used for resolving concurrency issues. The request fails if the version
    provided does not match the server version at the time of the request. If not provided,
    Square executes a blind write; potentially overwriting data from another
    write.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    A read-only timestamp in RFC 3339 format; presented in UTC.
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    A read-only timestamp in RFC 3339 format; presented in UTC.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class CatalogQueryExactParams(typing_extensions.TypedDict):
    """
    The query filter to return the search result by exact match of the specified attribute name and value.
    """

    attribute_name: str
    """
    The name of the attribute to be searched. Matching of the attribute name is exact.
    """

    attribute_value: str
    """
    The desired value of the search attribute. Matching of the attribute value is case insensitive and can be partial.
    For example, if a specified value of "sma", objects with the named attribute value of "Small", "small" are both matched.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class EventData(UncheckedBaseModel):
    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the affected object’s type.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the affected object.
    """

    deleted: typing.Optional[bool] = pydantic.Field(default=None)
    """
    This is true if the affected object has been deleted; otherwise, it's absent.
    """

    object: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    An object containing fields and values relevant to the event. It is absent if the affected object has been deleted.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...requests.bulk_delete_order_custom_attributes_request_delete_custom_attribute import (
    BulkDeleteOrderCustomAttributesRequestDeleteCustomAttributeParams,
)
from ...core.request_options import RequestOptions
from ...core.http_response import HttpResponse
from ...types.bulk_delete_order_custom_attributes_response import BulkDeleteOrderCustomAttributesResponse
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...requests.bulk_upsert_order_custom_attributes_request_upsert_custom_attribute import (
    BulkUpsertOrderCustomAttributesRequestUpsertCustomAttributeParams,
)
from ...types.bulk_upsert_order_custom_attributes_response import BulkUpsertOrderCustomAttributesResponse
from ...types.retrieve_order_custom_attribute_response import RetrieveOrderCustomAttributeResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...requests.custom_attribute import CustomAttributeParams
from ...types.upsert_order_custom_attribute_response import UpsertOrderCustomAttributeResponse
from ...types.delete_order_custom_attribute_response import DeleteOrderCustomAttributeResponse
from ...core.client_wrapper import AsyncClientWrapper
from ...core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawCustomAttributesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def batch_delete(
        self,
        *,
        values: typing.Dict[str, BulkDeleteOrderCustomAttributesRequestDeleteCustomAttributeParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[BulkDeleteOrderCustomAttributesResponse]:
        """
        Deletes order [custom attributes](entity:CustomAttribute) as a bulk operation.

        Use this endpoint to delete one or more custom attributes from one or more orders.
        A custom attribute is based on a custom attribute definition in a Square seller account.  (To create a
        custom attribute definition, use the [CreateOrderCustomAttributeDefinition](api-endpoint:OrderCustomAttributes-CreateOrderCustomAttributeDefinition) endpoint.)

        This `BulkDeleteOrderCustomAttributes` endpoint accepts a map of 1 to 25 individual delete
        requests and returns a map of individual delete responses. Each delete request has a unique ID
        and provides an order ID and custom attribute. Each delete response is returned with the ID
        of the corresponding request.

        To delete a custom attribute owned by another application, the `visibility` setting
        must be `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        values : typing.Dict[str, BulkDeleteOrderCustomAttributesRequestDeleteCustomAttributeParams]
            A map of requests that correspond to individual delete operations for custom attributes.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BulkDeleteOrderCustomAttributesResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/orders/custom-attributes/bulk-delete",
            method="POST",
            json={
                "values": convert_and_respect_annotation_metadata(
                    object_=values,
                    annotation=typing.Dict[str, BulkDeleteOrderCustomAttributesRequestDeleteCustomAttributeParams],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BulkDeleteOrderCustomAttributesResponse,
                    construct_type(
                        type_=BulkDeleteOrderCustomAttributesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def batch_upsert(
        self,
        *,
        values: typing.Dict[str, BulkUpsertOrderCustomAttributesRequestUpsertCustomAttributeParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[BulkUpsertOrderCustomAttributesResponse]:
        """
        Creates or updates order [custom attributes](entity:CustomAttribute) as a bulk operation.

        Use this endpoint to delete one or more custom attributes from one or more orders.
        A custom attribute is based on a custom attribute definition in a Square seller account.  (To create a
        custom attribute definition, use the [CreateOrderCustomAttributeDefinition](api-endpoint:OrderCustomAttributes-CreateOrderCustomAttributeDefinition) endpoint.)

        This `BulkUpsertOrderCustomAttributes` endpoint accepts a map of 1 to 25 individual upsert
        requests and returns a map of individual upsert responses. Each upsert request has a unique ID
        and provides an order ID and custom attribute. Each upsert response is returned with the ID
        of the corresponding request.

        To create or update a custom attribute owned by another application, the `visibility` setting
        must be `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        values : typing.Dict[str, BulkUpsertOrderCustomAttributesRequestUpsertCustomAttributeParams]
            A map of requests that correspond to individual upsert operations for custom attributes.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BulkUpsertOrderCustomAttributesResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/orders/custom-attributes/bulk-upsert",
            method="POST",
            json={
                "values": convert_and_respect_annotation_metadata(
                    object_=values,
                    annotation=typing.Dict[str, BulkUpsertOrderCustomAttributesRequestUpsertCustomAttributeParams],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BulkUpsertOrderCustomAttributesResponse,
                    construct_type(
                        type_=BulkUpsertOrderCustomAttributesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self,
        order_id: str,
        custom_attribute_key: str,
        *,
        version: typing.Optional[int] = None,
        with_definition: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[RetrieveOrderCustomAttributeResponse]:
        """
        Retrieves a [custom attribute](entity:CustomAttribute) associated with an order.

        You can use the `with_definition` query parameter to also retrieve the custom attribute definition
        in the same call.

        To retrieve a custom attribute owned by another application, the `visibility` setting must be
        `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        order_id : str
            The ID of the target [order](entity:Order).

        custom_attribute_key : str
            The key of the custom attribute to retrieve.  This key must match the key of an
            existing custom attribute definition.

        version : typing.Optional[int]
            To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control, include this optional field and specify the current version of the custom attribute.

        with_definition : typing.Optional[bool]
            Indicates whether to return the [custom attribute definition](entity:CustomAttributeDefinition) in the `definition` field of each
            custom attribute. Set this parameter to `true` to get the name and description of each custom attribute,
            information about the data type, or other definition details. The default value is `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RetrieveOrderCustomAttributeResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/orders/{jsonable_encoder(order_id)}/custom-attributes/{jsonable_encoder(custom_attribute_key)}",
            method="GET",
            params={
                "version": version,
                "with_definition": with_definition,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RetrieveOrderCustomAttributeResponse,
                    construct_type(
                        type_=RetrieveOrderCustomAttributeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def upsert(
        self,
        order_id: str,
        custom_attribute_key: str,
        *,
        custom_attribute: CustomAttributeParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpsertOrderCustomAttributeResponse]:
        """
        Creates or updates a [custom attribute](entity:CustomAttribute) for an order.

        Use this endpoint to set the value of a custom attribute for a specific order.
        A custom attribute is based on a custom attribute definition in a Square seller account. (To create a
        custom attribute definition, use the [CreateOrderCustomAttributeDefinition](api-endpoint:OrderCustomAttributes-CreateOrderCustomAttributeDefinition) endpoint.)

        To create or update a custom attribute owned by another application, the `visibility` setting
        must be `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        order_id : str
            The ID of the target [order](entity:Order).

        custom_attribute_key : str
            The key of the custom attribute to create or update.  This key must match the key
            of an existing custom attribute definition.

        custom_attribute : CustomAttributeParams
            The custom attribute to create or update, with the following fields:

            - `value`. This value must conform to the `schema` specified by the definition.
            For more information, see [Value data types](https://developer.squareup.com/docs/customer-custom-attributes-api/custom-attributes#value-data-types).

            - `version`. To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control, include this optional field and specify the current version of the custom attribute.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency.
            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpsertOrderCustomAttributeResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/orders/{jsonable_encoder(order_id)}/custom-attributes/{jsonable_encoder(custom_attribute_key)}",
            method="POST",
            json={
                "custom_attribute": convert_and_respect_annotation_metadata(
                    object_=custom_attribute, annotation=CustomAttributeParams, direction="write"
                ),
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpsertOrderCustomAttributeResponse,
                    construct_type(
                        type_=UpsertOrderCustomAttributeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self, order_id: str, custom_attribute_key: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeleteOrderCustomAttributeResponse]:
        """
        Deletes a [custom attribute](entity:CustomAttribute) associated with a customer profile.

        To delete a custom attribute owned by another application, the `visibility` setting must be
        `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        order_id : str
            The ID of the target [order](entity:Order).

        custom_attribute_key : str
            The key of the custom attribute to delete.  This key must match the key of an
            existing custom attribute definition.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeleteOrderCustomAttributeResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/orders/{jsonable_encoder(order_id)}/custom-attributes/{jsonable_encoder(custom_attribute_key)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteOrderCustomAttributeResponse,
                    construct_type(
                        type_=DeleteOrderCustomAttributeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawCustomAttributesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def batch_delete(
        self,
        *,
        values: typing.Dict[str, BulkDeleteOrderCustomAttributesRequestDeleteCustomAttributeParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[BulkDeleteOrderCustomAttributesResponse]:
        """
        Deletes order [custom attributes](entity:CustomAttribute) as a bulk operation.

        Use this endpoint to delete one or more custom attributes from one or more orders.
        A custom attribute is based on a custom attribute definition in a Square seller account.  (To create a
        custom attribute definition, use the [CreateOrderCustomAttributeDefinition](api-endpoint:OrderCustomAttributes-CreateOrderCustomAttributeDefinition) endpoint.)

        This `BulkDeleteOrderCustomAttributes` endpoint accepts a map of 1 to 25 individual delete
        requests and returns a map of individual delete responses. Each delete request has a unique ID
        and provides an order ID and custom attribute. Each delete response is returned with the ID
        of the corresponding request.

        To delete a custom attribute owned by another application, the `visibility` setting
        must be `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        values : typing.Dict[str, BulkDeleteOrderCustomAttributesRequestDeleteCustomAttributeParams]
            A map of requests that correspond to individual delete operations for custom attributes.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BulkDeleteOrderCustomAttributesResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/orders/custom-attributes/bulk-delete",
            method="POST",
            json={
                "values": convert_and_respect_annotation_metadata(
                    object_=values,
                    annotation=typing.Dict[str, BulkDeleteOrderCustomAttributesRequestDeleteCustomAttributeParams],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BulkDeleteOrderCustomAttributesResponse,
                    construct_type(
                        type_=BulkDeleteOrderCustomAttributesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def batch_upsert(
        self,
        *,
        values: typing.Dict[str, BulkUpsertOrderCustomAttributesRequestUpsertCustomAttributeParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[BulkUpsertOrderCustomAttributesResponse]:
        """
        Creates or updates order [custom attributes](entity:CustomAttribute) as a bulk operation.

        Use this endpoint to delete one or more custom attributes from one or more orders.
        A custom attribute is based on a custom attribute definition in a Square seller account.  (To create a
        custom attribute definition, use the [CreateOrderCustomAttributeDefinition](api-endpoint:OrderCustomAttributes-CreateOrderCustomAttributeDefinition) endpoint.)

        This `BulkUpsertOrderCustomAttributes` endpoint accepts a map of 1 to 25 individual upsert
        requests and returns a map of individual upsert responses. Each upsert request has a unique ID
        and provides an order ID and custom attribute. Each upsert response is returned with the ID
        of the corresponding request.

        To create or update a custom attribute owned by another application, the `visibility` setting
        must be `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        values : typing.Dict[str, BulkUpsertOrderCustomAttributesRequestUpsertCustomAttributeParams]
            A map of requests that correspond to individual upsert operations for custom attributes.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BulkUpsertOrderCustomAttributesResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/orders/custom-attributes/bulk-upsert",
            method="POST",
            json={
                "values": convert_and_respect_annotation_metadata(
                    object_=values,
                    annotation=typing.Dict[str, BulkUpsertOrderCustomAttributesRequestUpsertCustomAttributeParams],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BulkUpsertOrderCustomAttributesResponse,
                    construct_type(
                        type_=BulkUpsertOrderCustomAttributesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self,
        order_id: str,
        custom_attribute_key: str,
        *,
        version: typing.Optional[int] = None,
        with_definition: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[RetrieveOrderCustomAttributeResponse]:
        """
        Retrieves a [custom attribute](entity:CustomAttribute) associated with an order.

        You can use the `with_definition` query parameter to also retrieve the custom attribute definition
        in the same call.

        To retrieve a custom attribute owned by another application, the `visibility` setting must be
        `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        order_id : str
            The ID of the target [order](entity:Order).

        custom_attribute_key : str
            The key of the custom attribute to retrieve.  This key must match the key of an
            existing custom attribute definition.

        version : typing.Optional[int]
            To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control, include this optional field and specify the current version of the custom attribute.

        with_definition : typing.Optional[bool]
            Indicates whether to return the [custom attribute definition](entity:CustomAttributeDefinition) in the `definition` field of each
            custom attribute. Set this parameter to `true` to get the name and description of each custom attribute,
            information about the data type, or other definition details. The default value is `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RetrieveOrderCustomAttributeResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/orders/{jsonable_encoder(order_id)}/custom-attributes/{jsonable_encoder(custom_attribute_key)}",
            method="GET",
            params={
                "version": version,
                "with_definition": with_definition,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RetrieveOrderCustomAttributeResponse,
                    construct_type(
                        type_=RetrieveOrderCustomAttributeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def upsert(
        self,
        order_id: str,
        custom_attribute_key: str,
        *,
        custom_attribute: CustomAttributeParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpsertOrderCustomAttributeResponse]:
        """
        Creates or updates a [custom attribute](entity:CustomAttribute) for an order.

        Use this endpoint to set the value of a custom attribute for a specific order.
        A custom attribute is based on a custom attribute definition in a Square seller account. (To create a
        custom attribute definition, use the [CreateOrderCustomAttributeDefinition](api-endpoint:OrderCustomAttributes-CreateOrderCustomAttributeDefinition) endpoint.)

        To create or update a custom attribute owned by another application, the `visibility` setting
        must be `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        order_id : str
            The ID of the target [order](entity:Order).

        custom_attribute_key : str
            The key of the custom attribute to create or update.  This key must match the key
            of an existing custom attribute definition.

        custom_attribute : CustomAttributeParams
            The custom attribute to create or update, with the following fields:

            - `value`. This value must conform to the `schema` specified by the definition.
            For more information, see [Value data types](https://developer.squareup.com/docs/customer-custom-attributes-api/custom-attributes#value-data-types).

            - `version`. To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control, include this optional field and specify the current version of the custom attribute.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency.
            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpsertOrderCustomAttributeResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/orders/{jsonable_encoder(order_id)}/custom-attributes/{jsonable_encoder(custom_attribute_key)}",
            method="POST",
            json={
                "custom_attribute": convert_and_respect_annotation_metadata(
                    object_=custom_attribute, annotation=CustomAttributeParams, direction="write"
                ),
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpsertOrderCustomAttributeResponse,
                    construct_type(
                        type_=UpsertOrderCustomAttributeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, order_id: str, custom_attribute_key: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeleteOrderCustomAttributeResponse]:
        """
        Deletes a [custom attribute](entity:CustomAttribute) associated with a customer profile.

        To delete a custom attribute owned by another application, the `visibility` setting must be
        `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        order_id : str
            The ID of the target [order](entity:Order).

        custom_attribute_key : str
            The key of the custom attribute to delete.  This key must match the key of an
            existing custom attribute definition.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeleteOrderCustomAttributeResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/orders/{jsonable_encoder(order_id)}/custom-attributes/{jsonable_encoder(custom_attribute_key)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteOrderCustomAttributeResponse,
                    construct_type(
                        type_=DeleteOrderCustomAttributeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class DestinationDetailsExternalRefundDetails(UncheckedBaseModel):
    """
    Stores details about an external refund. Contains only non-confidential information.
    """

    type: str = pydantic.Field()
    """
    The type of external refund the seller paid to the buyer. It can be one of the
    following:
    - CHECK - Refunded using a physical check.
    - BANK_TRANSFER - Refunded using external bank transfer.
    - OTHER\_GIFT\_CARD - Refunded using a non-Square gift card.
    - CRYPTO - Refunded using a crypto currency.
    - SQUARE_CASH - Refunded using Square Cash App.
    - SOCIAL - Refunded using peer-to-peer payment applications.
    - EXTERNAL - A third-party application gathered this refund outside of Square.
    - EMONEY - Refunded using an E-money provider.
    - CARD - A credit or debit card that Square does not support.
    - STORED_BALANCE - Use for house accounts, store credit, and so forth.
    - FOOD_VOUCHER - Restaurant voucher provided by employers to employees to pay for meals
    - OTHER - A type not listed here.
    """

    source: str = pydantic.Field()
    """
    A description of the external refund source. For example,
    "Food Delivery Service".
    """

    source_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    An ID to associate the refund to its originating source.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams


class CancelPaymentByIdempotencyKeyResponseParams(typing_extensions.TypedDict):
    """
    Defines the response returned by
    [CancelPaymentByIdempotencyKey](api-endpoint:Payments-CancelPaymentByIdempotencyKey).
    On success, `errors` is empty.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .team_member_assigned_locations_assignment_type import TeamMemberAssignedLocationsAssignmentType
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class TeamMemberAssignedLocations(UncheckedBaseModel):
    """
    An object that represents a team member's assignment to locations.
    """

    assignment_type: typing.Optional[TeamMemberAssignedLocationsAssignmentType] = pydantic.Field(default=None)
    """
    The current assignment type of the team member.
    See [TeamMemberAssignedLocationsAssignmentType](#type-teammemberassignedlocationsassignmenttype) for possible values
    """

    location_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The explicit locations that the team member is assigned to.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .address import AddressParams
from .customer_preferences import CustomerPreferencesParams
from ..types.customer_creation_source import CustomerCreationSource
from .customer_tax_ids import CustomerTaxIdsParams


class CustomerParams(typing_extensions.TypedDict):
    """
    Represents a Square customer profile in the Customer Directory of a Square seller.
    """

    id: typing_extensions.NotRequired[str]
    """
    A unique Square-assigned ID for the customer profile.
    
    If you need this ID for an API request, use the ID returned when you created the customer profile or call the [SearchCustomers](api-endpoint:Customers-SearchCustomers) 
    or [ListCustomers](api-endpoint:Customers-ListCustomers) endpoint.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The timestamp when the customer profile was created, in RFC 3339 format.
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    The timestamp when the customer profile was last updated, in RFC 3339 format.
    """

    given_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The given name (that is, the first name) associated with the customer profile.
    """

    family_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The family name (that is, the last name) associated with the customer profile.
    """

    nickname: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A nickname for the customer profile.
    """

    company_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A business name associated with the customer profile.
    """

    email_address: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The email address associated with the customer profile.
    """

    address: typing_extensions.NotRequired[AddressParams]
    """
    The physical address associated with the customer profile.
    """

    phone_number: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The phone number associated with the customer profile.
    """

    birthday: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The birthday associated with the customer profile, in `YYYY-MM-DD` format. For example, `1998-09-21`
    represents September 21, 1998, and `0000-09-21` represents September 21 (without a birth year).
    """

    reference_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    An optional second ID used to associate the customer profile with an
    entity in another system.
    """

    note: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A custom note associated with the customer profile.
    """

    preferences: typing_extensions.NotRequired[CustomerPreferencesParams]
    """
    Represents general customer preferences.
    """

    creation_source: typing_extensions.NotRequired[CustomerCreationSource]
    """
    The method used to create the customer profile.
    See [CustomerCreationSource](#type-customercreationsource) for possible values
    """

    group_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    The IDs of [customer groups](entity:CustomerGroup) the customer belongs to.
    """

    segment_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    The IDs of [customer segments](entity:CustomerSegment) the customer belongs to.
    """

    version: typing_extensions.NotRequired[int]
    """
    The Square-assigned version number of the customer profile. The version number is incremented each time an update is committed to the customer profile, except for changes to customer segment membership.
    """

    tax_ids: typing_extensions.NotRequired[CustomerTaxIdsParams]
    """
    The tax ID associated with the customer profile. This field is present only for customers of sellers in EU countries or the United Kingdom. 
    For more information, see [Customer tax IDs](https://developer.squareup.com/docs/customers-api/what-it-does#customer-tax-ids).
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .money import Money
from .payment_options import PaymentOptions
from .device_checkout_options import DeviceCheckoutOptions
from .action_cancel_reason import ActionCancelReason
from .checkout_options_payment_type import CheckoutOptionsPaymentType
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class TerminalCheckout(UncheckedBaseModel):
    """
    Represents a checkout processed by the Square Terminal.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique ID for this `TerminalCheckout`.
    """

    amount_money: Money = pydantic.Field()
    """
    The amount of money (including the tax amount) that the Square Terminal device should try to collect.
    """

    reference_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    An optional user-defined reference ID that can be used to associate
    this `TerminalCheckout` to another entity in an external system. For example, an order
    ID generated by a third-party shopping cart. The ID is also associated with any payments
    used to complete the checkout.
    """

    note: typing.Optional[str] = pydantic.Field(default=None)
    """
    An optional note to associate with the checkout, as well as with any payments used to complete the checkout.
    Note: maximum 500 characters
    """

    order_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The reference to the Square order ID for the checkout request.
    """

    payment_options: typing.Optional[PaymentOptions] = pydantic.Field(default=None)
    """
    Payment-specific options for the checkout request.
    """

    device_options: DeviceCheckoutOptions = pydantic.Field()
    """
    Options to control the display and behavior of the Square Terminal device.
    """

    deadline_duration: typing.Optional[str] = pydantic.Field(default=None)
    """
    An RFC 3339 duration, after which the checkout is automatically canceled.
    A `TerminalCheckout` that is `PENDING` is automatically `CANCELED` and has a cancellation reason
    of `TIMED_OUT`.
    
    Default: 5 minutes from creation
    
    Maximum: 5 minutes
    """

    status: typing.Optional[str] = pydantic.Field(default=None)
    """
    The status of the `TerminalCheckout`.
    Options: `PENDING`, `IN_PROGRESS`, `CANCEL_REQUESTED`, `CANCELED`, `COMPLETED`
    """

    cancel_reason: typing.Optional[ActionCancelReason] = pydantic.Field(default=None)
    """
    The reason why `TerminalCheckout` is canceled. Present if the status is `CANCELED`.
    See [ActionCancelReason](#type-actioncancelreason) for possible values
    """

    payment_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of IDs for payments created by this `TerminalCheckout`.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The time when the `TerminalCheckout` was created, as an RFC 3339 timestamp.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The time when the `TerminalCheckout` was last updated, as an RFC 3339 timestamp.
    """

    app_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the application that created the checkout.
    """

    location_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The location of the device where the `TerminalCheckout` was directed.
    """

    payment_type: typing.Optional[CheckoutOptionsPaymentType] = pydantic.Field(default=None)
    """
    The type of payment the terminal should attempt to capture from. Defaults to `CARD_PRESENT`.
    See [CheckoutOptionsPaymentType](#type-checkoutoptionspaymenttype) for possible values
    """

    team_member_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    An optional ID of the team member associated with creating the checkout.
    """

    customer_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    An optional ID of the customer associated with the checkout.
    """

    app_fee_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The amount the developer is taking as a fee for facilitating the payment on behalf
    of the seller.
    
    The amount cannot be more than 90% of the total amount of the payment.
    
    The amount must be specified in the smallest denomination of the applicable currency (for example, US dollar amounts are specified in cents). For more information, see [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts).
    
    The fee currency code must match the currency associated with the seller that is accepting the payment. The application must be from a developer account in the same country and using the same currency code as the seller.
    
    For more information about the application fee scenario, see [Take Payments and Collect Fees](https://developer.squareup.com/docs/payments-api/take-payments-and-collect-fees).
    
    To set this field, PAYMENTS_WRITE_ADDITIONAL_RECIPIENTS OAuth permission is required. For more information, see [Permissions](https://developer.squareup.com/docs/payments-api/take-payments-and-collect-fees#permissions).
    """

    statement_description_identifier: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional additional payment information to include on the customer's card statement as
    part of the statement description. This can be, for example, an invoice number, ticket number,
    or short description that uniquely identifies the purchase.
    """

    tip_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The amount designated as a tip, in addition to `amount_money`. This may only be set for a
    checkout that has tipping disabled (`tip_settings.allow_tipping` is `false`).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .terminal_checkout import TerminalCheckoutParams


class CancelTerminalCheckoutResponseParams(typing_extensions.TypedDict):
    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information about errors encountered during the request.
    """

    checkout: typing_extensions.NotRequired[TerminalCheckoutParams]
    """
    The canceled `TerminalCheckout`.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
from .money import Money
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class GiftCardActivityImportReversal(UncheckedBaseModel):
    """
    Represents details about an `IMPORT_REVERSAL` [gift card activity type](entity:GiftCardActivityType).
    """

    amount_money: Money = pydantic.Field()
    """
    The amount of money cleared from the third-party gift card when 
    the import was reversed.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .custom_attribute import CustomAttributeParams
import typing
from .error import ErrorParams


class BulkUpsertMerchantCustomAttributesResponseMerchantCustomAttributeUpsertResponseParams(
    typing_extensions.TypedDict
):
    """
    Represents a response for an individual upsert request in a [BulkUpsertMerchantCustomAttributes](api-endpoint:MerchantCustomAttributes-BulkUpsertMerchantCustomAttributes) operation.
    """

    merchant_id: typing_extensions.NotRequired[str]
    """
    The ID of the merchant associated with the custom attribute.
    """

    custom_attribute: typing_extensions.NotRequired[CustomAttributeParams]
    """
    The new or updated custom attribute.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred while processing the individual request.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class OrderSource(UncheckedBaseModel):
    """
    Represents the origination details of an order.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name used to identify the place (physical or digital) that an order originates.
    If unset, the name defaults to the name of the application that created the order.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .shift_filter_status import ShiftFilterStatus
from .time_range import TimeRange
from .shift_workday import ShiftWorkday
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ShiftFilter(UncheckedBaseModel):
    """
    Defines a filter used in a search for `Shift` records. `AND` logic is
    used by Square's servers to apply each filter property specified.

    Deprecated at Square API version 2025-05-21. See the [migration notes](https://developer.squareup.com/docs/labor-api/what-it-does#migration-notes).
    """

    location_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Fetch shifts for the specified location.
    """

    employee_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Fetch shifts for the specified employees. DEPRECATED at version 2020-08-26. Use `team_member_ids` instead.
    """

    status: typing.Optional[ShiftFilterStatus] = pydantic.Field(default=None)
    """
    Fetch a `Shift` instance by `Shift.status`.
    See [ShiftFilterStatus](#type-shiftfilterstatus) for possible values
    """

    start: typing.Optional[TimeRange] = pydantic.Field(default=None)
    """
    Fetch `Shift` instances that start in the time range - Inclusive.
    """

    end: typing.Optional[TimeRange] = pydantic.Field(default=None)
    """
    Fetch the `Shift` instances that end in the time range - Inclusive.
    """

    workday: typing.Optional[ShiftWorkday] = pydantic.Field(default=None)
    """
    Fetch the `Shift` instances based on the workday date range.
    """

    team_member_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Fetch shifts for the specified team members. Replaced `employee_ids` at version "2020-08-26".
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

CatalogDiscountType = typing.Union[
    typing.Literal["FIXED_PERCENTAGE", "FIXED_AMOUNT", "VARIABLE_PERCENTAGE", "VARIABLE_AMOUNT"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawRefundsClient
from ..types.list_payment_refunds_request_sort_field import ListPaymentRefundsRequestSortField
from ..core.request_options import RequestOptions
from ..core.pagination import SyncPager
from ..types.payment_refund import PaymentRefund
from ..types.list_payment_refunds_response import ListPaymentRefundsResponse
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..requests.money import MoneyParams
from ..requests.destination_details_cash_refund_details import DestinationDetailsCashRefundDetailsParams
from ..requests.destination_details_external_refund_details import DestinationDetailsExternalRefundDetailsParams
from ..types.refund_payment_response import RefundPaymentResponse
from ..types.get_payment_refund_response import GetPaymentRefundResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawRefundsClient
from ..core.pagination import AsyncPager

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RefundsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawRefundsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawRefundsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawRefundsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        begin_time: typing.Optional[str] = None,
        end_time: typing.Optional[str] = None,
        sort_order: typing.Optional[str] = None,
        cursor: typing.Optional[str] = None,
        location_id: typing.Optional[str] = None,
        status: typing.Optional[str] = None,
        source_type: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        updated_at_begin_time: typing.Optional[str] = None,
        updated_at_end_time: typing.Optional[str] = None,
        sort_field: typing.Optional[ListPaymentRefundsRequestSortField] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[PaymentRefund]:
        """
        Retrieves a list of refunds for the account making the request.

        Results are eventually consistent, and new refunds or changes to refunds might take several
        seconds to appear.

        The maximum results per page is 100.

        Parameters
        ----------
        begin_time : typing.Optional[str]
            Indicates the start of the time range to retrieve each `PaymentRefund` for, in RFC 3339
            format.  The range is determined using the `created_at` field for each `PaymentRefund`.

            Default: The current time minus one year.

        end_time : typing.Optional[str]
            Indicates the end of the time range to retrieve each `PaymentRefund` for, in RFC 3339
            format.  The range is determined using the `created_at` field for each `PaymentRefund`.

            Default: The current time.

        sort_order : typing.Optional[str]
            The order in which results are listed by `PaymentRefund.created_at`:
            - `ASC` - Oldest to newest.
            - `DESC` - Newest to oldest (default).

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        location_id : typing.Optional[str]
            Limit results to the location supplied. By default, results are returned
            for all locations associated with the seller.

        status : typing.Optional[str]
            If provided, only refunds with the given status are returned.
            For a list of refund status values, see [PaymentRefund](entity:PaymentRefund).

            Default: If omitted, refunds are returned regardless of their status.

        source_type : typing.Optional[str]
            If provided, only returns refunds whose payments have the indicated source type.
            Current values include `CARD`, `BANK_ACCOUNT`, `WALLET`, `CASH`, and `EXTERNAL`.
            For information about these payment source types, see
            [Take Payments](https://developer.squareup.com/docs/payments-api/take-payments).

            Default: If omitted, refunds are returned regardless of the source type.

        limit : typing.Optional[int]
            The maximum number of results to be returned in a single page.

            It is possible to receive fewer results than the specified limit on a given page.

            If the supplied value is greater than 100, no more than 100 results are returned.

            Default: 100

        updated_at_begin_time : typing.Optional[str]
            Indicates the start of the time range to retrieve each `PaymentRefund` for, in RFC 3339
            format.  The range is determined using the `updated_at` field for each `PaymentRefund`.

            Default: If omitted, the time range starts at `begin_time`.

        updated_at_end_time : typing.Optional[str]
            Indicates the end of the time range to retrieve each `PaymentRefund` for, in RFC 3339
            format.  The range is determined using the `updated_at` field for each `PaymentRefund`.

            Default: The current time.

        sort_field : typing.Optional[ListPaymentRefundsRequestSortField]
            The field used to sort results by. The default is `CREATED_AT`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[PaymentRefund]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.refunds.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/refunds",
            method="GET",
            params={
                "begin_time": begin_time,
                "end_time": end_time,
                "sort_order": sort_order,
                "cursor": cursor,
                "location_id": location_id,
                "status": status,
                "source_type": source_type,
                "limit": limit,
                "updated_at_begin_time": updated_at_begin_time,
                "updated_at_end_time": updated_at_end_time,
                "sort_field": sort_field,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListPaymentRefundsResponse,
                    construct_type(
                        type_=ListPaymentRefundsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    begin_time=begin_time,
                    end_time=end_time,
                    sort_order=sort_order,
                    cursor=_parsed_next,
                    location_id=location_id,
                    status=status,
                    source_type=source_type,
                    limit=limit,
                    updated_at_begin_time=updated_at_begin_time,
                    updated_at_end_time=updated_at_end_time,
                    sort_field=sort_field,
                    request_options=request_options,
                )
                _items = _parsed_response.refunds
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def refund_payment(
        self,
        *,
        idempotency_key: str,
        amount_money: MoneyParams,
        app_fee_money: typing.Optional[MoneyParams] = OMIT,
        payment_id: typing.Optional[str] = OMIT,
        destination_id: typing.Optional[str] = OMIT,
        unlinked: typing.Optional[bool] = OMIT,
        location_id: typing.Optional[str] = OMIT,
        customer_id: typing.Optional[str] = OMIT,
        reason: typing.Optional[str] = OMIT,
        payment_version_token: typing.Optional[str] = OMIT,
        team_member_id: typing.Optional[str] = OMIT,
        cash_details: typing.Optional[DestinationDetailsCashRefundDetailsParams] = OMIT,
        external_details: typing.Optional[DestinationDetailsExternalRefundDetailsParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RefundPaymentResponse:
        """
        Refunds a payment. You can refund the entire payment amount or a
        portion of it. You can use this endpoint to refund a card payment or record a
        refund of a cash or external payment. For more information, see
        [Refund Payment](https://developer.squareup.com/docs/payments-api/refund-payments).

        Parameters
        ----------
        idempotency_key : str
             A unique string that identifies this `RefundPayment` request. The key can be any valid string
            but must be unique for every `RefundPayment` request.

            Keys are limited to a max of 45 characters - however, the number of allowed characters might be
            less than 45, if multi-byte characters are used.

            For more information, see [Idempotency](https://developer.squareup.com/docs/working-with-apis/idempotency).

        amount_money : MoneyParams
            The amount of money to refund.

            This amount cannot be more than the `total_money` value of the payment minus the total
            amount of all previously completed refunds for this payment.

            This amount must be specified in the smallest denomination of the applicable currency
            (for example, US dollar amounts are specified in cents). For more information, see
            [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts).

            The currency code must match the currency associated with the business
            that is charging the card.

        app_fee_money : typing.Optional[MoneyParams]
            The amount of money the developer contributes to help cover the refunded amount.
            This amount is specified in the smallest denomination of the applicable currency (for example,
            US dollar amounts are specified in cents).

            The value cannot be more than the `amount_money`.

            You can specify this parameter in a refund request only if the same parameter was also included
            when taking the payment. This is part of the application fee scenario the API supports. For more
            information, see [Take Payments and Collect Fees](https://developer.squareup.com/docs/payments-api/take-payments-and-collect-fees).

            To set this field, `PAYMENTS_WRITE_ADDITIONAL_RECIPIENTS` OAuth permission is required.
            For more information, see [Permissions](https://developer.squareup.com/docs/payments-api/take-payments-and-collect-fees#permissions).

        payment_id : typing.Optional[str]
            The unique ID of the payment being refunded.
            Required when unlinked=false, otherwise must not be set.

        destination_id : typing.Optional[str]
            The ID indicating where funds will be refunded to. Required for unlinked refunds. For more
            information, see [Process an Unlinked Refund](https://developer.squareup.com/docs/refunds-api/unlinked-refunds).

            For refunds linked to Square payments, `destination_id` is usually omitted; in this case, funds
            will be returned to the original payment source. The field may be specified in order to request
            a cross-method refund to a gift card. For more information,
            see [Cross-method refunds to gift cards](https://developer.squareup.com/docs/payments-api/refund-payments#cross-method-refunds-to-gift-cards).

        unlinked : typing.Optional[bool]
            Indicates that the refund is not linked to a Square payment.
            If set to true, `destination_id` and `location_id` must be supplied while `payment_id` must not
            be provided.

        location_id : typing.Optional[str]
            The location ID associated with the unlinked refund.
            Required for requests specifying `unlinked=true`.
            Otherwise, if included when `unlinked=false`, will throw an error.

        customer_id : typing.Optional[str]
            The [Customer](entity:Customer) ID of the customer associated with the refund.
            This is required if the `destination_id` refers to a card on file created using the Cards
            API. Only allowed when `unlinked=true`.

        reason : typing.Optional[str]
            A description of the reason for the refund.

        payment_version_token : typing.Optional[str]
             Used for optimistic concurrency. This opaque token identifies the current `Payment`
            version that the caller expects. If the server has a different version of the Payment,
            the update fails and a response with a VERSION_MISMATCH error is returned.
            If the versions match, or the field is not provided, the refund proceeds as normal.

        team_member_id : typing.Optional[str]
            An optional [TeamMember](entity:TeamMember) ID to associate with this refund.

        cash_details : typing.Optional[DestinationDetailsCashRefundDetailsParams]
            Additional details required when recording an unlinked cash refund (`destination_id` is CASH).

        external_details : typing.Optional[DestinationDetailsExternalRefundDetailsParams]
            Additional details required when recording an unlinked external refund
            (`destination_id` is EXTERNAL).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RefundPaymentResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.refunds.refund_payment(
            idempotency_key="9b7f2dcf-49da-4411-b23e-a2d6af21333a",
            amount_money={"amount": 1000, "currency": "USD"},
            app_fee_money={"amount": 10, "currency": "USD"},
            payment_id="R2B3Z8WMVt3EAmzYWLZvz7Y69EbZY",
            reason="Example",
        )
        """
        response = self._raw_client.refund_payment(
            idempotency_key=idempotency_key,
            amount_money=amount_money,
            app_fee_money=app_fee_money,
            payment_id=payment_id,
            destination_id=destination_id,
            unlinked=unlinked,
            location_id=location_id,
            customer_id=customer_id,
            reason=reason,
            payment_version_token=payment_version_token,
            team_member_id=team_member_id,
            cash_details=cash_details,
            external_details=external_details,
            request_options=request_options,
        )
        return response.data

    def get(
        self, refund_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetPaymentRefundResponse:
        """
        Retrieves a specific refund using the `refund_id`.

        Parameters
        ----------
        refund_id : str
            The unique ID for the desired `PaymentRefund`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetPaymentRefundResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.refunds.get(
            refund_id="refund_id",
        )
        """
        response = self._raw_client.get(refund_id, request_options=request_options)
        return response.data


class AsyncRefundsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawRefundsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawRefundsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawRefundsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        begin_time: typing.Optional[str] = None,
        end_time: typing.Optional[str] = None,
        sort_order: typing.Optional[str] = None,
        cursor: typing.Optional[str] = None,
        location_id: typing.Optional[str] = None,
        status: typing.Optional[str] = None,
        source_type: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        updated_at_begin_time: typing.Optional[str] = None,
        updated_at_end_time: typing.Optional[str] = None,
        sort_field: typing.Optional[ListPaymentRefundsRequestSortField] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[PaymentRefund]:
        """
        Retrieves a list of refunds for the account making the request.

        Results are eventually consistent, and new refunds or changes to refunds might take several
        seconds to appear.

        The maximum results per page is 100.

        Parameters
        ----------
        begin_time : typing.Optional[str]
            Indicates the start of the time range to retrieve each `PaymentRefund` for, in RFC 3339
            format.  The range is determined using the `created_at` field for each `PaymentRefund`.

            Default: The current time minus one year.

        end_time : typing.Optional[str]
            Indicates the end of the time range to retrieve each `PaymentRefund` for, in RFC 3339
            format.  The range is determined using the `created_at` field for each `PaymentRefund`.

            Default: The current time.

        sort_order : typing.Optional[str]
            The order in which results are listed by `PaymentRefund.created_at`:
            - `ASC` - Oldest to newest.
            - `DESC` - Newest to oldest (default).

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        location_id : typing.Optional[str]
            Limit results to the location supplied. By default, results are returned
            for all locations associated with the seller.

        status : typing.Optional[str]
            If provided, only refunds with the given status are returned.
            For a list of refund status values, see [PaymentRefund](entity:PaymentRefund).

            Default: If omitted, refunds are returned regardless of their status.

        source_type : typing.Optional[str]
            If provided, only returns refunds whose payments have the indicated source type.
            Current values include `CARD`, `BANK_ACCOUNT`, `WALLET`, `CASH`, and `EXTERNAL`.
            For information about these payment source types, see
            [Take Payments](https://developer.squareup.com/docs/payments-api/take-payments).

            Default: If omitted, refunds are returned regardless of the source type.

        limit : typing.Optional[int]
            The maximum number of results to be returned in a single page.

            It is possible to receive fewer results than the specified limit on a given page.

            If the supplied value is greater than 100, no more than 100 results are returned.

            Default: 100

        updated_at_begin_time : typing.Optional[str]
            Indicates the start of the time range to retrieve each `PaymentRefund` for, in RFC 3339
            format.  The range is determined using the `updated_at` field for each `PaymentRefund`.

            Default: If omitted, the time range starts at `begin_time`.

        updated_at_end_time : typing.Optional[str]
            Indicates the end of the time range to retrieve each `PaymentRefund` for, in RFC 3339
            format.  The range is determined using the `updated_at` field for each `PaymentRefund`.

            Default: The current time.

        sort_field : typing.Optional[ListPaymentRefundsRequestSortField]
            The field used to sort results by. The default is `CREATED_AT`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[PaymentRefund]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.refunds.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/refunds",
            method="GET",
            params={
                "begin_time": begin_time,
                "end_time": end_time,
                "sort_order": sort_order,
                "cursor": cursor,
                "location_id": location_id,
                "status": status,
                "source_type": source_type,
                "limit": limit,
                "updated_at_begin_time": updated_at_begin_time,
                "updated_at_end_time": updated_at_end_time,
                "sort_field": sort_field,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListPaymentRefundsResponse,
                    construct_type(
                        type_=ListPaymentRefundsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    begin_time=begin_time,
                    end_time=end_time,
                    sort_order=sort_order,
                    cursor=_parsed_next,
                    location_id=location_id,
                    status=status,
                    source_type=source_type,
                    limit=limit,
                    updated_at_begin_time=updated_at_begin_time,
                    updated_at_end_time=updated_at_end_time,
                    sort_field=sort_field,
                    request_options=request_options,
                )
                _items = _parsed_response.refunds
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def refund_payment(
        self,
        *,
        idempotency_key: str,
        amount_money: MoneyParams,
        app_fee_money: typing.Optional[MoneyParams] = OMIT,
        payment_id: typing.Optional[str] = OMIT,
        destination_id: typing.Optional[str] = OMIT,
        unlinked: typing.Optional[bool] = OMIT,
        location_id: typing.Optional[str] = OMIT,
        customer_id: typing.Optional[str] = OMIT,
        reason: typing.Optional[str] = OMIT,
        payment_version_token: typing.Optional[str] = OMIT,
        team_member_id: typing.Optional[str] = OMIT,
        cash_details: typing.Optional[DestinationDetailsCashRefundDetailsParams] = OMIT,
        external_details: typing.Optional[DestinationDetailsExternalRefundDetailsParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RefundPaymentResponse:
        """
        Refunds a payment. You can refund the entire payment amount or a
        portion of it. You can use this endpoint to refund a card payment or record a
        refund of a cash or external payment. For more information, see
        [Refund Payment](https://developer.squareup.com/docs/payments-api/refund-payments).

        Parameters
        ----------
        idempotency_key : str
             A unique string that identifies this `RefundPayment` request. The key can be any valid string
            but must be unique for every `RefundPayment` request.

            Keys are limited to a max of 45 characters - however, the number of allowed characters might be
            less than 45, if multi-byte characters are used.

            For more information, see [Idempotency](https://developer.squareup.com/docs/working-with-apis/idempotency).

        amount_money : MoneyParams
            The amount of money to refund.

            This amount cannot be more than the `total_money` value of the payment minus the total
            amount of all previously completed refunds for this payment.

            This amount must be specified in the smallest denomination of the applicable currency
            (for example, US dollar amounts are specified in cents). For more information, see
            [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts).

            The currency code must match the currency associated with the business
            that is charging the card.

        app_fee_money : typing.Optional[MoneyParams]
            The amount of money the developer contributes to help cover the refunded amount.
            This amount is specified in the smallest denomination of the applicable currency (for example,
            US dollar amounts are specified in cents).

            The value cannot be more than the `amount_money`.

            You can specify this parameter in a refund request only if the same parameter was also included
            when taking the payment. This is part of the application fee scenario the API supports. For more
            information, see [Take Payments and Collect Fees](https://developer.squareup.com/docs/payments-api/take-payments-and-collect-fees).

            To set this field, `PAYMENTS_WRITE_ADDITIONAL_RECIPIENTS` OAuth permission is required.
            For more information, see [Permissions](https://developer.squareup.com/docs/payments-api/take-payments-and-collect-fees#permissions).

        payment_id : typing.Optional[str]
            The unique ID of the payment being refunded.
            Required when unlinked=false, otherwise must not be set.

        destination_id : typing.Optional[str]
            The ID indicating where funds will be refunded to. Required for unlinked refunds. For more
            information, see [Process an Unlinked Refund](https://developer.squareup.com/docs/refunds-api/unlinked-refunds).

            For refunds linked to Square payments, `destination_id` is usually omitted; in this case, funds
            will be returned to the original payment source. The field may be specified in order to request
            a cross-method refund to a gift card. For more information,
            see [Cross-method refunds to gift cards](https://developer.squareup.com/docs/payments-api/refund-payments#cross-method-refunds-to-gift-cards).

        unlinked : typing.Optional[bool]
            Indicates that the refund is not linked to a Square payment.
            If set to true, `destination_id` and `location_id` must be supplied while `payment_id` must not
            be provided.

        location_id : typing.Optional[str]
            The location ID associated with the unlinked refund.
            Required for requests specifying `unlinked=true`.
            Otherwise, if included when `unlinked=false`, will throw an error.

        customer_id : typing.Optional[str]
            The [Customer](entity:Customer) ID of the customer associated with the refund.
            This is required if the `destination_id` refers to a card on file created using the Cards
            API. Only allowed when `unlinked=true`.

        reason : typing.Optional[str]
            A description of the reason for the refund.

        payment_version_token : typing.Optional[str]
             Used for optimistic concurrency. This opaque token identifies the current `Payment`
            version that the caller expects. If the server has a different version of the Payment,
            the update fails and a response with a VERSION_MISMATCH error is returned.
            If the versions match, or the field is not provided, the refund proceeds as normal.

        team_member_id : typing.Optional[str]
            An optional [TeamMember](entity:TeamMember) ID to associate with this refund.

        cash_details : typing.Optional[DestinationDetailsCashRefundDetailsParams]
            Additional details required when recording an unlinked cash refund (`destination_id` is CASH).

        external_details : typing.Optional[DestinationDetailsExternalRefundDetailsParams]
            Additional details required when recording an unlinked external refund
            (`destination_id` is EXTERNAL).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RefundPaymentResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.refunds.refund_payment(
                idempotency_key="9b7f2dcf-49da-4411-b23e-a2d6af21333a",
                amount_money={"amount": 1000, "currency": "USD"},
                app_fee_money={"amount": 10, "currency": "USD"},
                payment_id="R2B3Z8WMVt3EAmzYWLZvz7Y69EbZY",
                reason="Example",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.refund_payment(
            idempotency_key=idempotency_key,
            amount_money=amount_money,
            app_fee_money=app_fee_money,
            payment_id=payment_id,
            destination_id=destination_id,
            unlinked=unlinked,
            location_id=location_id,
            customer_id=customer_id,
            reason=reason,
            payment_version_token=payment_version_token,
            team_member_id=team_member_id,
            cash_details=cash_details,
            external_details=external_details,
            request_options=request_options,
        )
        return response.data

    async def get(
        self, refund_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetPaymentRefundResponse:
        """
        Retrieves a specific refund using the `refund_id`.

        Parameters
        ----------
        refund_id : str
            The unique ID for the desired `PaymentRefund`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetPaymentRefundResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.refunds.get(
                refund_id="refund_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(refund_id, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class CatalogEcomSeoDataParams(typing_extensions.TypedDict):
    """
    SEO data for for a seller's Square Online store.
    """

    page_title: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The SEO title used for the Square Online store.
    """

    page_description: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The SEO description used for the Square Online store.
    """

    permalink: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The SEO permalink used for the Square Online store.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .error import Error
import pydantic
from .loyalty_promotion import LoyaltyPromotion
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ListLoyaltyPromotionsResponse(UncheckedBaseModel):
    """
    Represents a [ListLoyaltyPromotions](api-endpoint:Loyalty-ListLoyaltyPromotions) response.
    One of `loyalty_promotions`, an empty object, or `errors` is present in the response.
    If additional results are available, the `cursor` field is also present along with `loyalty_promotions`.
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Any errors that occurred during the request.
    """

    loyalty_promotions: typing.Optional[typing.List[LoyaltyPromotion]] = pydantic.Field(default=None)
    """
    The retrieved loyalty promotions.
    """

    cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    The cursor to use in your next call to this endpoint to retrieve the next page of results
    for your original request. This field is present only if the request succeeded and additional
    results are available. For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .money import MoneyParams
from ..types.dispute_reason import DisputeReason
from ..types.dispute_state import DisputeState
from .disputed_payment import DisputedPaymentParams
from ..types.card_brand import CardBrand


class DisputeParams(typing_extensions.TypedDict):
    """
    Represents a [dispute](https://developer.squareup.com/docs/disputes-api/overview) a cardholder initiated with their bank.
    """

    dispute_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The unique ID for this `Dispute`, generated by Square.
    """

    id: typing_extensions.NotRequired[str]
    """
    The unique ID for this `Dispute`, generated by Square.
    """

    amount_money: typing_extensions.NotRequired[MoneyParams]
    """
    The disputed amount, which can be less than the total transaction amount.
    For instance, if multiple items were purchased but the cardholder only initiates a dispute over some of the items.
    """

    reason: typing_extensions.NotRequired[DisputeReason]
    """
    The reason why the cardholder initiated the dispute.
    See [DisputeReason](#type-disputereason) for possible values
    """

    state: typing_extensions.NotRequired[DisputeState]
    """
    The current state of this dispute.
    See [DisputeState](#type-disputestate) for possible values
    """

    due_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The deadline by which the seller must respond to the dispute, in [RFC 3339 format](https://developer.squareup.com/docs/build-basics/common-data-types/working-with-dates).
    """

    disputed_payment: typing_extensions.NotRequired[DisputedPaymentParams]
    """
    The payment challenged in this dispute.
    """

    evidence_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    The IDs of the evidence associated with the dispute.
    """

    card_brand: typing_extensions.NotRequired[CardBrand]
    """
    The card brand used in the disputed payment.
    See [CardBrand](#type-cardbrand) for possible values
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The timestamp when the dispute was created, in RFC 3339 format.
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    The timestamp when the dispute was last updated, in RFC 3339 format.
    """

    brand_dispute_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the dispute in the card brand system, generated by the card brand.
    """

    reported_date: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The timestamp when the dispute was reported, in RFC 3339 format.
    """

    reported_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The timestamp when the dispute was reported, in RFC 3339 format.
    """

    version: typing_extensions.NotRequired[int]
    """
    The current version of the `Dispute`.
    """

    location_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the location where the dispute originated.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .catalog_object import CatalogObjectParams


class GetCatalogObjectResponseParams(typing_extensions.TypedDict):
    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    object: typing_extensions.NotRequired[CatalogObjectParams]
    """
    The `CatalogObject`s returned.
    """

    related_objects: typing_extensions.NotRequired[typing.Sequence[CatalogObjectParams]]
    """
    A list of `CatalogObject`s referenced by the object in the `object` field.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .cash_drawer_shift_event import CashDrawerShiftEventParams


class ListCashDrawerShiftEventsResponseParams(typing_extensions.TypedDict):
    cursor: typing_extensions.NotRequired[str]
    """
    Opaque cursor for fetching the next page. Cursor is not present in
    the last page of results.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    cash_drawer_shift_events: typing_extensions.NotRequired[typing.Sequence[CashDrawerShiftEventParams]]
    """
    All of the events (payments, refunds, etc.) for a cash drawer during
    the shift.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.employee_status import EmployeeStatus


class EmployeeParams(typing_extensions.TypedDict):
    """
    An employee object that is used by the external API.

    DEPRECATED at version 2020-08-26. Replaced by [TeamMember](entity:TeamMember).
    """

    id: typing_extensions.NotRequired[str]
    """
    UUID for this object.
    """

    first_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The employee's first name.
    """

    last_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The employee's last name.
    """

    email: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The employee's email address
    """

    phone_number: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The employee's phone number in E.164 format, i.e. "+12125554250"
    """

    location_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    A list of location IDs where this employee has access to.
    """

    status: typing_extensions.NotRequired[EmployeeStatus]
    """
    Specifies the status of the employees being fetched.
    See [EmployeeStatus](#type-employeestatus) for possible values
    """

    is_owner: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Whether this employee is the owner of the merchant. Each merchant
    has one owner employee, and that employee has full authority over
    the account.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    A read-only timestamp in RFC 3339 format.
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    A read-only timestamp in RFC 3339 format.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .loyalty_event import LoyaltyEventParams


class AdjustLoyaltyPointsResponseParams(typing_extensions.TypedDict):
    """
    Represents an [AdjustLoyaltyPoints](api-endpoint:Loyalty-AdjustLoyaltyPoints) request.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    event: typing_extensions.NotRequired[LoyaltyEventParams]
    """
    The resulting event data for the adjustment.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .scheduled_shift_sort_field import ScheduledShiftSortField
import pydantic
from .sort_order import SortOrder
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ScheduledShiftSort(UncheckedBaseModel):
    """
    Defines sort criteria for a [SearchScheduledShifts](api-endpoint:Labor-SearchScheduledShifts)
    request.
    """

    field: typing.Optional[ScheduledShiftSortField] = pydantic.Field(default=None)
    """
    The field to sort on. The default value is `START_AT`.
    See [ScheduledShiftSortField](#type-scheduledshiftsortfield) for possible values
    """

    order: typing.Optional[SortOrder] = pydantic.Field(default=None)
    """
    The order in which results are returned. The default value is `ASC`.
    See [SortOrder](#type-sortorder) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class OrderSourceParams(typing_extensions.TypedDict):
    """
    Represents the origination details of an order.
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name used to identify the place (physical or digital) that an order originates.
    If unset, the name defaults to the name of the application that created the order.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.checkout_location_settings_branding_header_type import CheckoutLocationSettingsBrandingHeaderType
import typing
from ..types.checkout_location_settings_branding_button_shape import CheckoutLocationSettingsBrandingButtonShape


class CheckoutLocationSettingsBrandingParams(typing_extensions.TypedDict):
    header_type: typing_extensions.NotRequired[CheckoutLocationSettingsBrandingHeaderType]
    """
    Show the location logo on the checkout page.
    See [HeaderType](#type-headertype) for possible values
    """

    button_color: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The HTML-supported hex color for the button on the checkout page (for example, "#FFFFFF").
    """

    button_shape: typing_extensions.NotRequired[CheckoutLocationSettingsBrandingButtonShape]
    """
    The shape of the button on the checkout page.
    See [ButtonShape](#type-buttonshape) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogItemOptionForItem(UncheckedBaseModel):
    """
     An option that can be assigned to an item.
    For example, a t-shirt item may offer a color option or a size option.
    """

    item_option_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique id of the item option, used to form the dimensions of the item option matrix in a specified order.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .publish_scheduled_shift_response import PublishScheduledShiftResponseParams
from .error import ErrorParams


class BulkPublishScheduledShiftsResponseParams(typing_extensions.TypedDict):
    """
    Represents a [BulkPublishScheduledShifts](api-endpoint:Labor-BulkPublishScheduledShifts) response.
    Either `scheduled_shifts` or `errors` is present in the response.
    """

    responses: typing_extensions.NotRequired[typing.Dict[str, PublishScheduledShiftResponseParams]]
    """
    A map of key-value pairs that represent responses for individual publish requests.
    The order of responses might differ from the order in which the requests were provided.
    
    - Each key is the scheduled shift ID that was specified for a publish request.
    - Each value is the corresponding response. If the request succeeds, the value is the
    published scheduled shift. If the request fails, the value is an `errors` array containing
    any errors that occurred while processing the request.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any top-level errors that prevented the bulk operation from succeeding.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .get_vendor_response import GetVendorResponseParams


class BatchGetVendorsResponseParams(typing_extensions.TypedDict):
    """
    Represents an output from a call to [BulkRetrieveVendors](api-endpoint:Vendors-BulkRetrieveVendors).
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    responses: typing_extensions.NotRequired[typing.Dict[str, GetVendorResponseParams]]
    """
    The set of [RetrieveVendorResponse](entity:RetrieveVendorResponse) objects encapsulating successfully retrieved [Vendor](entity:Vendor)
    objects or error responses for failed attempts. The set is represented by 
    a collection of `Vendor`-ID/`Vendor`-object or `Vendor`-ID/error-object pairs.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .cash_app_details import CashAppDetails
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class DigitalWalletDetails(UncheckedBaseModel):
    """
    Additional details about `WALLET` type payments. Contains only non-confidential information.
    """

    status: typing.Optional[str] = pydantic.Field(default=None)
    """
    The status of the `WALLET` payment. The status can be `AUTHORIZED`, `CAPTURED`, `VOIDED`, or
    `FAILED`.
    """

    brand: typing.Optional[str] = pydantic.Field(default=None)
    """
    The brand used for the `WALLET` payment. The brand can be `CASH_APP`, `PAYPAY`, `ALIPAY`,
    `RAKUTEN_PAY`, `AU_PAY`, `D_BARAI`, `MERPAY`, `WECHAT_PAY` or `UNKNOWN`.
    """

    cash_app_details: typing.Optional[CashAppDetails] = pydantic.Field(default=None)
    """
    Brand-specific details for payments with the `brand` of `CASH_APP`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .money import Money
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class OrderMoneyAmounts(UncheckedBaseModel):
    """
    A collection of various money amounts.
    """

    total_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The total money.
    """

    tax_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The money associated with taxes.
    """

    discount_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The money associated with discounts.
    """

    tip_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The money associated with tips.
    """

    service_charge_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The money associated with service charges.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogQueryText(UncheckedBaseModel):
    """
    The query filter to return the search result whose searchable attribute values contain all of the specified keywords or tokens, independent of the token order or case.
    """

    keywords: typing.List[str] = pydantic.Field()
    """
    A list of 1, 2, or 3 search keywords. Keywords with fewer than 3 alphanumeric characters are ignored.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.gift_card_type import GiftCardType
from ..types.gift_card_gan_source import GiftCardGanSource
from ..types.gift_card_status import GiftCardStatus
from .money import MoneyParams
import typing


class GiftCardParams(typing_extensions.TypedDict):
    """
    Represents a Square gift card.
    """

    id: typing_extensions.NotRequired[str]
    """
    The Square-assigned ID of the gift card.
    """

    type: GiftCardType
    """
    The gift card type.
    See [Type](#type-type) for possible values
    """

    gan_source: typing_extensions.NotRequired[GiftCardGanSource]
    """
    The source that generated the gift card account number (GAN). The default value is `SQUARE`.
    See [GANSource](#type-gansource) for possible values
    """

    state: typing_extensions.NotRequired[GiftCardStatus]
    """
    The current gift card state.
    See [Status](#type-status) for possible values
    """

    balance_money: typing_extensions.NotRequired[MoneyParams]
    """
    The current gift card balance. This balance is always greater than or equal to zero.
    """

    gan: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The gift card account number (GAN). Buyers can use the GAN to make purchases or check 
    the gift card balance.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The timestamp when the gift card was created, in RFC 3339 format. 
    In the case of a digital gift card, it is the time when you create a card 
    (using the Square Point of Sale application, Seller Dashboard, or Gift Cards API).  
    In the case of a plastic gift card, it is the time when Square associates the card with the 
    seller at the time of activation.
    """

    customer_ids: typing_extensions.NotRequired[typing.Sequence[str]]
    """
    The IDs of the [customer profiles](entity:Customer) to whom this gift card is linked.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .publish_scheduled_shift_response import PublishScheduledShiftResponse
import pydantic
from .error import Error
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class BulkPublishScheduledShiftsResponse(UncheckedBaseModel):
    """
    Represents a [BulkPublishScheduledShifts](api-endpoint:Labor-BulkPublishScheduledShifts) response.
    Either `scheduled_shifts` or `errors` is present in the response.
    """

    responses: typing.Optional[typing.Dict[str, PublishScheduledShiftResponse]] = pydantic.Field(default=None)
    """
    A map of key-value pairs that represent responses for individual publish requests.
    The order of responses might differ from the order in which the requests were provided.
    
    - Each key is the scheduled shift ID that was specified for a publish request.
    - Each value is the corresponding response. If the request succeeds, the value is the
    published scheduled shift. If the request fails, the value is an `errors` array containing
    any errors that occurred while processing the request.
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Any top-level errors that prevented the bulk operation from succeeding.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
import typing_extensions
import typing_extensions
import typing
import typing

if typing.TYPE_CHECKING:
    from .catalog_object import CatalogObjectParams


class CatalogItemOptionParams(typing_extensions.TypedDict):
    """
    A group of variations for a `CatalogItem`.
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The item option's display name for the seller. Must be unique across
    all item options. This is a searchable attribute for use in applicable query filters.
    """

    display_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The item option's display name for the customer. This is a searchable attribute for use in applicable query filters.
    """

    description: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The item option's human-readable description. Displayed in the Square
    Point of Sale app for the seller and in the Online Store or on receipts for
    the buyer. This is a searchable attribute for use in applicable query filters.
    """

    show_colors: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    If true, display colors for entries in `values` when present.
    """

    values: typing_extensions.NotRequired[typing.Optional[typing.Sequence["CatalogObjectParams"]]]
    """
    A list of CatalogObjects containing the
    `CatalogItemOptionValue`s for this item.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams


class DisableEventsResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields that are included in the response body of
    a request to the [DisableEvents](api-endpoint:Events-DisableEvents) endpoint.

    Note: if there are errors processing the request, the events field will not be
    present.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information on errors encountered during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.fulfillment_type import FulfillmentType
from ..types.fulfillment_state import FulfillmentState
from ..types.fulfillment_fulfillment_line_item_application import FulfillmentFulfillmentLineItemApplication
from .fulfillment_fulfillment_entry import FulfillmentFulfillmentEntryParams
from .fulfillment_pickup_details import FulfillmentPickupDetailsParams
from .fulfillment_shipment_details import FulfillmentShipmentDetailsParams
from .fulfillment_delivery_details import FulfillmentDeliveryDetailsParams


class FulfillmentParams(typing_extensions.TypedDict):
    """
    Contains details about how to fulfill this order.
    Orders can only be created with at most one fulfillment using the API.
    However, orders returned by the Orders API might contain multiple fulfillments because sellers can create multiple fulfillments using Square products such as Square Online.
    """

    uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A unique ID that identifies the fulfillment only within this order.
    """

    type: typing_extensions.NotRequired[FulfillmentType]
    """
    The type of the fulfillment.
    See [FulfillmentType](#type-fulfillmenttype) for possible values
    """

    state: typing_extensions.NotRequired[FulfillmentState]
    """
    The state of the fulfillment.
    See [FulfillmentState](#type-fulfillmentstate) for possible values
    """

    line_item_application: typing_extensions.NotRequired[FulfillmentFulfillmentLineItemApplication]
    """
    Describes what order line items this fulfillment applies to.
    It can be `ALL` or `ENTRY_LIST` with a supplied list of fulfillment entries.
    See [FulfillmentFulfillmentLineItemApplication](#type-fulfillmentfulfillmentlineitemapplication) for possible values
    """

    entries: typing_extensions.NotRequired[typing.Sequence[FulfillmentFulfillmentEntryParams]]
    """
    A list of entries pertaining to the fulfillment of an order. Each entry must reference
    a valid `uid` for an order line item in the `line_item_uid` field, as well as a `quantity` to
    fulfill.
    
    Multiple entries can reference the same line item `uid`, as long as the total quantity among
    all fulfillment entries referencing a single line item does not exceed the quantity of the
    order's line item itself.
    
    An order cannot be marked as `COMPLETED` before all fulfillments are `COMPLETED`,
    `CANCELED`, or `FAILED`. Fulfillments can be created and completed independently
    before order completion.
    """

    metadata: typing_extensions.NotRequired[typing.Optional[typing.Dict[str, typing.Optional[str]]]]
    """
    Application-defined data attached to this fulfillment. Metadata fields are intended
    to store descriptive references or associations with an entity in another system or store brief
    information about the object. Square does not process this field; it only stores and returns it
    in relevant API calls. Do not use metadata to store any sensitive information (such as personally
    identifiable information or card details).
    
    Keys written by applications must be 60 characters or less and must be in the character set
    `[a-zA-Z0-9_-]`. Entries can also include metadata generated by Square. These keys are prefixed
    with a namespace, separated from the key with a ':' character.
    
    Values have a maximum length of 255 characters.
    
    An application can have up to 10 entries per metadata field.
    
    Entries written by applications are private and can only be read or modified by the same
    application.
    
    For more information, see [Metadata](https://developer.squareup.com/docs/build-basics/metadata).
    """

    pickup_details: typing_extensions.NotRequired[FulfillmentPickupDetailsParams]
    """
    Contains details for a pickup fulfillment. These details are required when the fulfillment
    type is `PICKUP`.
    """

    shipment_details: typing_extensions.NotRequired[FulfillmentShipmentDetailsParams]
    """
    Contains details for a shipment fulfillment. These details are required when the fulfillment type
    is `SHIPMENT`.
    
    A shipment fulfillment's relationship to fulfillment `state`:
    `PROPOSED`: A shipment is requested.
    `RESERVED`: Fulfillment in progress. Shipment processing.
    `PREPARED`: Shipment packaged. Shipping label created.
    `COMPLETED`: Package has been shipped.
    `CANCELED`: Shipment has been canceled.
    `FAILED`: Shipment has failed.
    """

    delivery_details: typing_extensions.NotRequired[FulfillmentDeliveryDetailsParams]
    """
    Describes delivery details of an order fulfillment.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class BulkPublishScheduledShiftsData(UncheckedBaseModel):
    """
    Represents options for an individual publish request in a
    [BulkPublishScheduledShifts](api-endpoint:Labor-BulkPublishScheduledShifts)
    operation, provided as the value in a key-value pair.
    """

    version: typing.Optional[int] = pydantic.Field(default=None)
    """
    The current version of the scheduled shift, used to enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
    control. If the provided version doesn't match the server version, the request fails.
    If omitted, Square executes a blind write, potentially overwriting data from another publish request.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .custom_attribute import CustomAttributeParams
from .error import ErrorParams


class ListOrderCustomAttributesResponseParams(typing_extensions.TypedDict):
    """
    Represents a response from listing order custom attributes.
    """

    custom_attributes: typing_extensions.NotRequired[typing.Sequence[CustomAttributeParams]]
    """
    The retrieved custom attributes. If no custom attribute are found, Square returns an empty object (`{}`).
    """

    cursor: typing_extensions.NotRequired[str]
    """
    The cursor to provide in your next call to this endpoint to retrieve the next page of results for your original request. 
    This field is present only if the request succeeded and additional results are available.
    For more information, see [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination).
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .tender_card_details_status import TenderCardDetailsStatus
import pydantic
from .card import Card
from .tender_card_details_entry_method import TenderCardDetailsEntryMethod
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class TenderCardDetails(UncheckedBaseModel):
    """
    Represents additional details of a tender with `type` `CARD` or `SQUARE_GIFT_CARD`
    """

    status: typing.Optional[TenderCardDetailsStatus] = pydantic.Field(default=None)
    """
    The credit card payment's current state (such as `AUTHORIZED` or
    `CAPTURED`). See [TenderCardDetailsStatus](entity:TenderCardDetailsStatus)
    for possible values.
    See [TenderCardDetailsStatus](#type-tendercarddetailsstatus) for possible values
    """

    card: typing.Optional[Card] = pydantic.Field(default=None)
    """
    The credit card's non-confidential details.
    """

    entry_method: typing.Optional[TenderCardDetailsEntryMethod] = pydantic.Field(default=None)
    """
    The method used to enter the card's details for the transaction.
    See [TenderCardDetailsEntryMethod](#type-tendercarddetailsentrymethod) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .inventory_change import InventoryChangeParams


class BatchChangeInventoryRequestParams(typing_extensions.TypedDict):
    idempotency_key: str
    """
    A client-supplied, universally unique identifier (UUID) for the
    request.
    
    See [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) in the
    [API Development 101](https://developer.squareup.com/docs/buildbasics) section for more
    information.
    """

    changes: typing_extensions.NotRequired[typing.Optional[typing.Sequence[InventoryChangeParams]]]
    """
    The set of physical counts and inventory adjustments to be made.
    Changes are applied based on the client-supplied timestamp and may be sent
    out of order.
    """

    ignore_unchanged_counts: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether the current physical count should be ignored if
    the quantity is unchanged since the last physical count. Default: `true`.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SaveCardOptions(UncheckedBaseModel):
    """
    Describes save-card action fields.
    """

    customer_id: str = pydantic.Field()
    """
    The square-assigned ID of the customer linked to the saved card.
    """

    card_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The id of the created card-on-file.
    """

    reference_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    An optional user-defined reference ID that can be used to associate
    this `Card` to another entity in an external system. For example, a customer
    ID generated by a third-party system.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .order import OrderParams
from .error import ErrorParams


class BatchGetOrdersResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields that are included in the response body of
    a request to the `BatchRetrieveOrders` endpoint.
    """

    orders: typing_extensions.NotRequired[typing.Sequence[OrderParams]]
    """
    The requested orders. This will omit any requested orders that do not exist.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .signature_image import SignatureImageParams


class SignatureOptionsParams(typing_extensions.TypedDict):
    title: str
    """
    The title text to display in the signature capture flow on the Terminal.
    """

    body: str
    """
    The body text to display in the signature capture flow on the Terminal.
    """

    signature: typing_extensions.NotRequired[typing.Sequence[SignatureImageParams]]
    """
    An image representation of the collected signature.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .loyalty_event import LoyaltyEventParams


class SearchLoyaltyEventsResponseParams(typing_extensions.TypedDict):
    """
    A response that contains loyalty events that satisfy the search
    criteria, in order by the `created_at` date.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    events: typing_extensions.NotRequired[typing.Sequence[LoyaltyEventParams]]
    """
    The loyalty events that satisfy the search criteria.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    The pagination cursor to be used in a subsequent 
    request. If empty, this is the final response. 
    For more information, 
    see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .risk_evaluation_risk_level import RiskEvaluationRiskLevel
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class RiskEvaluation(UncheckedBaseModel):
    """
    Represents fraud risk information for the associated payment.

    When you take a payment through Square's Payments API (using the `CreatePayment`
    endpoint), Square evaluates it and assigns a risk level to the payment. Sellers
    can use this information to determine the course of action (for example,
    provide the goods/services or refund the payment).
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when payment risk was evaluated, in RFC 3339 format.
    """

    risk_level: typing.Optional[RiskEvaluationRiskLevel] = pydantic.Field(default=None)
    """
    The risk level associated with the payment
    See [RiskEvaluationRiskLevel](#type-riskevaluationrisklevel) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .vendor import Vendor
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class UpdateVendorRequest(UncheckedBaseModel):
    """
    Represents an input to a call to [UpdateVendor](api-endpoint:Vendors-UpdateVendor).
    """

    idempotency_key: typing.Optional[str] = pydantic.Field(default=None)
    """
    A client-supplied, universally unique identifier (UUID) for the
    request.
    
    See [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) in the
    [API Development 101](https://developer.squareup.com/docs/buildbasics) section for more
    information.
    """

    vendor: Vendor = pydantic.Field()
    """
    The specified [Vendor](entity:Vendor) to be updated.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions


class SnippetParams(typing_extensions.TypedDict):
    """
    Represents the snippet that is added to a Square Online site. The snippet code is injected into the `head` element of all pages on the site, except for checkout pages.
    """

    id: typing_extensions.NotRequired[str]
    """
    The Square-assigned ID for the snippet.
    """

    site_id: typing_extensions.NotRequired[str]
    """
    The ID of the site that contains the snippet.
    """

    content: str
    """
    The snippet code, which can contain valid HTML, JavaScript, or both.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The timestamp of when the snippet was initially added to the site, in RFC 3339 format.
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    The timestamp of when the snippet was last updated on the site, in RFC 3339 format.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class LoyaltyEventLoyaltyAccountFilterParams(typing_extensions.TypedDict):
    """
    Filter events by loyalty account.
    """

    loyalty_account_id: str
    """
    The ID of the [loyalty account](entity:LoyaltyAccount) associated with loyalty events.
    """



# This file was auto-generated by Fern from our API Definition.

from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawEmployeeWagesClient
import typing
from ...core.request_options import RequestOptions
from ...core.pagination import SyncPager
from ...types.employee_wage import EmployeeWage
from ...types.list_employee_wages_response import ListEmployeeWagesResponse
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...types.get_employee_wage_response import GetEmployeeWageResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawEmployeeWagesClient
from ...core.pagination import AsyncPager


class EmployeeWagesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawEmployeeWagesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawEmployeeWagesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawEmployeeWagesClient
        """
        return self._raw_client

    def list(
        self,
        *,
        employee_id: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[EmployeeWage]:
        """
        Returns a paginated list of `EmployeeWage` instances for a business.

        Parameters
        ----------
        employee_id : typing.Optional[str]
            Filter the returned wages to only those that are associated with the specified employee.

        limit : typing.Optional[int]
            The maximum number of `EmployeeWage` results to return per page. The number can range between
            1 and 200. The default is 200.

        cursor : typing.Optional[str]
            A pointer to the next page of `EmployeeWage` results to fetch.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[EmployeeWage]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.labor.employee_wages.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/labor/employee-wages",
            method="GET",
            params={
                "employee_id": employee_id,
                "limit": limit,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListEmployeeWagesResponse,
                    construct_type(
                        type_=ListEmployeeWagesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    employee_id=employee_id,
                    limit=limit,
                    cursor=_parsed_next,
                    request_options=request_options,
                )
                _items = _parsed_response.employee_wages
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetEmployeeWageResponse:
        """
        Returns a single `EmployeeWage` specified by `id`.

        Parameters
        ----------
        id : str
            The UUID for the `EmployeeWage` being retrieved.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetEmployeeWageResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.labor.employee_wages.get(
            id="id",
        )
        """
        response = self._raw_client.get(id, request_options=request_options)
        return response.data


class AsyncEmployeeWagesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawEmployeeWagesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawEmployeeWagesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawEmployeeWagesClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        employee_id: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[EmployeeWage]:
        """
        Returns a paginated list of `EmployeeWage` instances for a business.

        Parameters
        ----------
        employee_id : typing.Optional[str]
            Filter the returned wages to only those that are associated with the specified employee.

        limit : typing.Optional[int]
            The maximum number of `EmployeeWage` results to return per page. The number can range between
            1 and 200. The default is 200.

        cursor : typing.Optional[str]
            A pointer to the next page of `EmployeeWage` results to fetch.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[EmployeeWage]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.labor.employee_wages.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/labor/employee-wages",
            method="GET",
            params={
                "employee_id": employee_id,
                "limit": limit,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListEmployeeWagesResponse,
                    construct_type(
                        type_=ListEmployeeWagesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    employee_id=employee_id,
                    limit=limit,
                    cursor=_parsed_next,
                    request_options=request_options,
                )
                _items = _parsed_response.employee_wages
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetEmployeeWageResponse:
        """
        Returns a single `EmployeeWage` specified by `id`.

        Parameters
        ----------
        id : str
            The UUID for the `EmployeeWage` being retrieved.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetEmployeeWageResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.labor.employee_wages.get(
                id="id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(id, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..types.gift_card_activity_deactivate_reason import GiftCardActivityDeactivateReason


class GiftCardActivityDeactivateParams(typing_extensions.TypedDict):
    """
    Represents details about a `DEACTIVATE` [gift card activity type](entity:GiftCardActivityType).
    """

    reason: GiftCardActivityDeactivateReason
    """
    The reason the gift card was deactivated.
    See [Reason](#type-reason) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .appointment_segment import AppointmentSegment
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Availability(UncheckedBaseModel):
    """
    Defines an appointment slot that encapsulates the appointment segments, location and starting time available for booking.
    """

    start_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The RFC 3339 timestamp specifying the beginning time of the slot available for booking.
    """

    location_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the location available for booking.
    """

    appointment_segments: typing.Optional[typing.List[AppointmentSegment]] = pydantic.Field(default=None)
    """
    The list of appointment segments available for booking
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .money import MoneyParams


class TimecardWageParams(typing_extensions.TypedDict):
    """
    The hourly wage rate used to compensate a team member for a [timecard](entity:Timecard).
    """

    title: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name of the job performed during this timecard.
    """

    hourly_rate: typing_extensions.NotRequired[MoneyParams]
    """
    Can be a custom-set hourly wage or the calculated effective hourly
    wage based on the annual wage and hours worked per week.
    """

    job_id: typing_extensions.NotRequired[str]
    """
    The ID of the [job](entity:Job) performed for this timecard. Square
    labor-reporting UIs might group timecards together by ID.
    """

    tip_eligible: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Whether team members are eligible for tips when working this job.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .card import CardParams
import typing


class DestinationDetailsCardRefundDetailsParams(typing_extensions.TypedDict):
    card: typing_extensions.NotRequired[CardParams]
    """
    The card's non-confidential details.
    """

    entry_method: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The method used to enter the card's details for the refund. The method can be
    `KEYED`, `SWIPED`, `EMV`, `ON_FILE`, or `CONTACTLESS`.
    """

    auth_result_code: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The authorization code provided by the issuer when a refund is approved.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .location_booking_profile import LocationBookingProfileParams
import typing
from .error import ErrorParams


class RetrieveLocationBookingProfileResponseParams(typing_extensions.TypedDict):
    location_booking_profile: typing_extensions.NotRequired[LocationBookingProfileParams]
    """
    The requested location booking profile.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class InvoiceAcceptedPaymentMethods(UncheckedBaseModel):
    """
    The payment methods that customers can use to pay an [invoice](entity:Invoice) on the Square-hosted invoice payment page.
    """

    card: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether credit card or debit card payments are accepted. The default value is `false`.
    """

    square_gift_card: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether Square gift card payments are accepted. The default value is `false`.
    """

    bank_account: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether ACH bank transfer payments are accepted. The default value is `false`.
    """

    buy_now_pay_later: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether Afterpay (also known as Clearpay) payments are accepted. The default value is `false`.
    
    This option is allowed only for invoices that have a single payment request of the `BALANCE` type. This payment method is
    supported if the seller account accepts Afterpay payments and the seller location is in a country where Afterpay
    invoice payments are supported. As a best practice, consider enabling an additional payment method when allowing
    `buy_now_pay_later` payments. For more information, including detailed requirements and processing limits, see
    [Buy Now Pay Later payments with Afterpay](https://developer.squareup.com/docs/invoices-api/overview#buy-now-pay-later).
    """

    cash_app_pay: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether Cash App payments are accepted. The default value is `false`.
    
    This payment method is supported only for seller [locations](entity:Location) in the United States.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..core.http_response import HttpResponse
from ..types.retrieve_location_settings_response import RetrieveLocationSettingsResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..requests.checkout_location_settings import CheckoutLocationSettingsParams
from ..types.update_location_settings_response import UpdateLocationSettingsResponse
from ..core.serialization import convert_and_respect_annotation_metadata
from ..types.retrieve_merchant_settings_response import RetrieveMerchantSettingsResponse
from ..requests.checkout_merchant_settings import CheckoutMerchantSettingsParams
from ..types.update_merchant_settings_response import UpdateMerchantSettingsResponse
from ..core.client_wrapper import AsyncClientWrapper
from ..core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawCheckoutClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def retrieve_location_settings(
        self, location_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[RetrieveLocationSettingsResponse]:
        """
        Retrieves the location-level settings for a Square-hosted checkout page.

        Parameters
        ----------
        location_id : str
            The ID of the location for which to retrieve settings.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RetrieveLocationSettingsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/online-checkout/location-settings/{jsonable_encoder(location_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RetrieveLocationSettingsResponse,
                    construct_type(
                        type_=RetrieveLocationSettingsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_location_settings(
        self,
        location_id: str,
        *,
        location_settings: CheckoutLocationSettingsParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateLocationSettingsResponse]:
        """
        Updates the location-level settings for a Square-hosted checkout page.

        Parameters
        ----------
        location_id : str
            The ID of the location for which to retrieve settings.

        location_settings : CheckoutLocationSettingsParams
            Describe your updates using the `location_settings` object. Make sure it contains only the fields that have changed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateLocationSettingsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/online-checkout/location-settings/{jsonable_encoder(location_id)}",
            method="PUT",
            json={
                "location_settings": convert_and_respect_annotation_metadata(
                    object_=location_settings, annotation=CheckoutLocationSettingsParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateLocationSettingsResponse,
                    construct_type(
                        type_=UpdateLocationSettingsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def retrieve_merchant_settings(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[RetrieveMerchantSettingsResponse]:
        """
        Retrieves the merchant-level settings for a Square-hosted checkout page.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RetrieveMerchantSettingsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/online-checkout/merchant-settings",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RetrieveMerchantSettingsResponse,
                    construct_type(
                        type_=RetrieveMerchantSettingsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_merchant_settings(
        self,
        *,
        merchant_settings: CheckoutMerchantSettingsParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateMerchantSettingsResponse]:
        """
        Updates the merchant-level settings for a Square-hosted checkout page.

        Parameters
        ----------
        merchant_settings : CheckoutMerchantSettingsParams
            Describe your updates using the `merchant_settings` object. Make sure it contains only the fields that have changed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateMerchantSettingsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/online-checkout/merchant-settings",
            method="PUT",
            json={
                "merchant_settings": convert_and_respect_annotation_metadata(
                    object_=merchant_settings, annotation=CheckoutMerchantSettingsParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateMerchantSettingsResponse,
                    construct_type(
                        type_=UpdateMerchantSettingsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawCheckoutClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def retrieve_location_settings(
        self, location_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[RetrieveLocationSettingsResponse]:
        """
        Retrieves the location-level settings for a Square-hosted checkout page.

        Parameters
        ----------
        location_id : str
            The ID of the location for which to retrieve settings.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RetrieveLocationSettingsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/online-checkout/location-settings/{jsonable_encoder(location_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RetrieveLocationSettingsResponse,
                    construct_type(
                        type_=RetrieveLocationSettingsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_location_settings(
        self,
        location_id: str,
        *,
        location_settings: CheckoutLocationSettingsParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateLocationSettingsResponse]:
        """
        Updates the location-level settings for a Square-hosted checkout page.

        Parameters
        ----------
        location_id : str
            The ID of the location for which to retrieve settings.

        location_settings : CheckoutLocationSettingsParams
            Describe your updates using the `location_settings` object. Make sure it contains only the fields that have changed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateLocationSettingsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/online-checkout/location-settings/{jsonable_encoder(location_id)}",
            method="PUT",
            json={
                "location_settings": convert_and_respect_annotation_metadata(
                    object_=location_settings, annotation=CheckoutLocationSettingsParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateLocationSettingsResponse,
                    construct_type(
                        type_=UpdateLocationSettingsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def retrieve_merchant_settings(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[RetrieveMerchantSettingsResponse]:
        """
        Retrieves the merchant-level settings for a Square-hosted checkout page.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RetrieveMerchantSettingsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/online-checkout/merchant-settings",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RetrieveMerchantSettingsResponse,
                    construct_type(
                        type_=RetrieveMerchantSettingsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_merchant_settings(
        self,
        *,
        merchant_settings: CheckoutMerchantSettingsParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateMerchantSettingsResponse]:
        """
        Updates the merchant-level settings for a Square-hosted checkout page.

        Parameters
        ----------
        merchant_settings : CheckoutMerchantSettingsParams
            Describe your updates using the `merchant_settings` object. Make sure it contains only the fields that have changed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateMerchantSettingsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/online-checkout/merchant-settings",
            method="PUT",
            json={
                "merchant_settings": convert_and_respect_annotation_metadata(
                    object_=merchant_settings, annotation=CheckoutMerchantSettingsParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateMerchantSettingsResponse,
                    construct_type(
                        type_=UpdateMerchantSettingsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.currency import Currency


class V1MoneyParams(typing_extensions.TypedDict):
    amount: typing_extensions.NotRequired[typing.Optional[int]]
    """
    Amount in the lowest denominated value of this Currency. E.g. in USD
    these are cents, in JPY they are Yen (which do not have a 'cent' concept).
    """

    currency_code: typing_extensions.NotRequired[Currency]
    """
    
    See [Currency](#type-currency) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.search_events_sort_field import SearchEventsSortField
from ..types.sort_order import SortOrder


class SearchEventsSortParams(typing_extensions.TypedDict):
    """
    Criteria to sort events by.
    """

    field: typing_extensions.NotRequired[SearchEventsSortField]
    """
    Sort events by event types.
    See [SearchEventsSortField](#type-searcheventssortfield) for possible values
    """

    order: typing_extensions.NotRequired[SortOrder]
    """
    The order to use for sorting the events.
    See [SortOrder](#type-sortorder) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .money import MoneyParams


class AdditionalRecipientParams(typing_extensions.TypedDict):
    """
    Represents an additional recipient (other than the merchant) receiving a portion of this tender.
    """

    location_id: str
    """
    The location ID for a recipient (other than the merchant) receiving a portion of this tender.
    """

    description: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The description of the additional recipient.
    """

    amount_money: MoneyParams
    """
    The amount of money distributed to the recipient.
    """

    receivable_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The unique ID for the RETIRED `AdditionalRecipientReceivable` object. This field should be empty for any `AdditionalRecipient` objects created after the retirement.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class LoyaltyPromotionIncentivePointsMultiplierDataParams(typing_extensions.TypedDict):
    """
    Represents the metadata for a `POINTS_MULTIPLIER` type of [loyalty promotion incentive](entity:LoyaltyPromotionIncentive).
    """

    points_multiplier: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The multiplier used to calculate the number of points earned each time the promotion
    is triggered. For example, suppose a purchase qualifies for 5 points from the base loyalty program.
    If the purchase also qualifies for a `POINTS_MULTIPLIER` promotion incentive with a `points_multiplier`
    of 3, the buyer earns a total of 15 points (5 program points x 3 promotion multiplier = 15 points).
    
    DEPRECATED at version 2023-08-16. Replaced by the `multiplier` field.
    
    One of the following is required when specifying a points multiplier:
    - (Recommended) The `multiplier` field.
    - This deprecated `points_multiplier` field. If provided in the request, Square also returns `multiplier`
    with the equivalent value.
    """

    multiplier: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The multiplier used to calculate the number of points earned each time the promotion is triggered,
    specified as a string representation of a decimal. Square supports multipliers up to 10x, with three
    point precision for decimal multipliers. For example, suppose a purchase qualifies for 4 points from the
    base loyalty program. If the purchase also qualifies for a `POINTS_MULTIPLIER` promotion incentive with a
    `multiplier` of "1.5", the buyer earns a total of 6 points (4 program points x 1.5 promotion multiplier = 6 points).
    Fractional points are dropped.
    
    One of the following is required when specifying a points multiplier:
    - (Recommended) This `multiplier` field.
    - The deprecated `points_multiplier` field. If provided in the request, Square also returns `multiplier`
    with the equivalent value.
    """
